
clear all
set more off 


***   directories
local dir = "...\replication"


***   =50 if imputed, =0 if not imputed
local thre = 50

***   regressors: censoring probabilities
local var_prob "s_13 x_0_9 x_10_19 x_20_29 x_30 g"

***   regressors: quantile regressions
local var_qreg "s s2 x x2 g"


*=======================================================================
* Stage 1: Quantile imputation
*=======================================================================

* seed for random selection of quantiles in the imputation procedure  
set seed 12345

*---------------------
* for-loop: years
*---------------------
foreach t of numlist 1979/2012 {
	
	*---------------------
	* for-loop: states
	*---------------------
	foreach j of num 1/50 {
		
		*===========================
		* Read indiviual-level data 
		*===========================
		use "`dir'\data\sample_`t'.dta", clear
		
		* Select state 
		keep if sid == `j'
		
		*=======================================================================
		* Step 1: Probability of censoring due to minimum wage and top-coding
		*=======================================================================
		di "step 1"
		merge m:1 year sid `var_prob' using "`dir'\data\prob\prob0\pr_`t'_`j'.dta"
		keep if sid == `j'
		
		*=======================================================================
		* Steps 2 and 3: Quantile regressions
		*=======================================================================
		di "step 2"
		* Probability of being censored
		sum   pr_c1 if d_miss == 0
		local pr_c1_max = r(max)
		sum   pr_c2 if d_miss == 0
		local pr_c2_max = r(max)
		* 4 types of censoring
		gen       case_step1 = 1 if `pr_c1_max' == 0 & `pr_c2_max' == 0
		replace   case_step1 = 2 if `pr_c1_max' == 0 & `pr_c2_max' ~= 0
		replace   case_step1 = 3 if `pr_c1_max' ~= 0 & `pr_c2_max' == 0
		replace   case_step1 = 4 if `pr_c1_max' ~= 0 & `pr_c2_max' ~= 0
		label var case_step1 "censoring type"
		
		*---------------------
		* for-loop: quantiles
		*---------------------
		foreach Q of num 4/97{
			
			local q = `Q' / 100
			
			* Subsample selection:
			* The probability of being top-coded is pr_c1(x) := Pr{y_max <= y|x}
			*  For q-th quantile regression drop if (1 - q) <= pr_c1(x).  
			
			if `pr_c1_max' == 0 {
							
				if `pr_c2_max' == 0 {
				
					*-----------------------------------------------------------
					* Case 1: No censoring  
					*-----------------------------------------------------------
					
					* Quantile regression w/o censoring using whole sample
					qreg lnw `var_qreg' if d_miss == 0 [pw = wgt], q(`q') wlsiter(1)
					
				}
				else {
					
					*-----------------------------------------------------------
					* Case 2: Right-censoring due to top coding
					*-----------------------------------------------------------
					
					*======================================
					* Step 2: Censored Quantile Regression
					*======================================
					* Probability of not being censored
					gen  no_pr_c2 = 1 - pr_c2 if d_miss == 0 
					* Subsample selection 1:
					* The quantile of interest should fall inside the range of
					*  probabilities of not being censored.
					gen  set2     = (`q' < no_pr_c2)  if d_miss == 0
					
					* Subsample selection 2 (refinement):
					* Reduce the probability of not being right-censored by
					*  the 0.1th quantiles of empirical probability in set2
					*  to accommodate possible specification errors.
					* Note that if the probability of not being censored is
					*  almost one, keep all observations.					
					centile no_pr_c2 if d_miss == 0 & set2 == 1, c(10)
					local   no_pc2 = r(c_1)
					
					* Stop dropping observations if the probability of not being
					*  censored is very large above "0.999".
					if `no_pc2' < 0.999 {
						replace set2 = 0 if d_miss == 0 & set2 == 1 & no_pr_c2 < `no_pc2'
					}
					
					* Quantile regression with selected sample
					qreg lnw `var_qreg' if d_miss == 0 & set2 == 1 [pw = wgt], q(`q') wlsiter(2)
					
					* Fitted values
					predict double yhat2 if d_miss == 0, xb
					
					* Value of the objective function in step 2
					gen      resid2 = lnw - yhat2 if d_miss == 0
					replace  resid2 = lnw - lntop if d_miss == 0 & lntop < yhat2 
					gen      vecR2   = ( `q' - 1 * (resid2 <= 0) ) * resid2 if d_miss == 0
					sum      vecR2 [aw = wgt]  if d_miss == 0 /* <== weighting is needed */
					local    obj2   = r(sum)
					drop     resid2 vecR2
					
					*=====================================
					* Step 3: Censored Quantile Regression
					*=====================================
					* Subsample selection 3:
					* Keep observations for which fitted values (i.e., predicted 
					*  probability of being top-coded) is below 0.97th quantile
					*  in selected sample.
					centile yhat2 if d_miss == 0 & yhat2 < lntop, c(97)
					local   y_cut2 = r(c_1)
					gen     set3   = (yhat2 < `y_cut2') if d_miss == 0
					
					* Quantile regression with selected sample
					qreg lnw `var_qreg' if d_miss == 0 & set3 == 1 [pw = wgt], q(`q') wlsiter(1)
					
					* Fitted values 
					predict double yhat3 if d_miss == 0, xb
					
					* Value of the objective function in step 3
					gen      resid3 = lnw - yhat3 if d_miss == 0
					replace  resid3 = lnw - lntop if d_miss == 0 & lntop < yhat3 
					gen      vecR3   = ( `q' - 1 * (resid3 <= 0) ) * resid3  if d_miss == 0
					sum      vecR3 [aw = wgt]  if d_miss == 0 /* weighting is needed */
					local    obj3   = r(sum)
					drop     resid3 vecR3
					
					* Check if the model fit improves in step 3. If it does not,
					* redo step 2. 
					if `obj2' < `obj3' {
						
						* Quantile regression with "set2"
						qreg lnw `var_qreg' if d_miss == 0 & set2 == 1 [pw = wgt], q(`q') wlsiter(2)
						
					}
					drop no_pr_c2 set2 set3 yhat2 yhat3
					
				}
			}
			else {
				
				if `pr_c2_max' == 0 {
					
					*-----------------------------------------------------------
					* Case 3: Left-censoring due to minimum wage
					*-----------------------------------------------------------
					
					*=====================================
					* Step 2: Censored quantile regression
					*=====================================
					* Subsample selection 1: 
					* The quantile of interest should fall inside the
					*  range of probabilities of not being censored.
					gen set2 = (pr_c1 < `q') if d_miss == 0
					
					* Subsample selection 2 (refinement):
					* Reduce the probability of not being right-censored by
					*  the 0.1th quantiles of empirical probability in set2
					*  to accommodate possible specification errors.
					* Note that if the probability of not being censored is
					*  almost one, keep all observations.
					centile pr_c1 if d_miss == 0 & set2 == 1, c(10)
					local   pc2 = r(c_1)
					
					* Stop dropping observations if the probability of being
					*  censored is very small above "0.001".
					if `pc2' > 0.001 {
						replace set2 = 0 if d_miss == 0 & set2 == 1 & pr_c1 < `pc2'
					}
					
					* Quantile regression with selected sample 
					qreg lnw `var_qreg' if d_miss == 0 & set2 == 1 [pw = wgt], q(`q') wlsiter(2)
					
					* Fitted values 
					predict double yhat2 if d_miss == 0, xb
					
					* Value of the objective function in step 2
					gen      resid2 = lnw - yhat2 if d_miss == 0
					replace  resid2 = lnw - lnmw  if d_miss == 0 & yhat2 < lnmw 
					gen      vecR2   = ( `q' - 1 * (resid2 <= 0) ) * resid2  if d_miss == 0
					sum      vecR2 [aw = wgt]  if d_miss == 0   /* <== weighting is needed */
					local    obj2   = r(sum)
					drop     resid2 vecR2
					
					*======================================
					* Step 3: Censored quantile regression
					*======================================
					* Subsample selection 3:
					* Keep observations for which fitted values (i.e., predicted 
					*  probability of being censored at the minimum wage) is
					*  above 0.03th quantile in selected sample.
					centile yhat2 if d_miss == 0 & lnmw < yhat2, c(3)  /* 3 */
					local   y_cut2 = r(c_1)
					gen     set3   = (`y_cut2' < yhat2)  if d_miss == 0
					
					* Quantile regression with selected sample 
					qreg lnw `var_qreg' if d_miss == 0 & set3 == 1 [pw = wgt], q(`q') wlsiter(1)
					
					* Fitted values 
					predict double yhat3 if d_miss == 0, xb
					
					* Value of the objective function in step 3
					gen      resid3 = lnw - yhat3 if d_miss == 0
					replace  resid3 = lnw - lnmw  if d_miss == 0 & yhat3 < lnmw 
					gen      vecR3  = ( `q' - 1 * (resid3 <= 0) ) * resid3 if d_miss == 0 
					sum      vecR3 [aw = wgt]  if d_miss == 0 /* weighting is needed */
					local    obj3   = r(sum)
					drop     resid3 vecR3
					
					* Check if the model fit improves in step 3. If it does not,
					* redo step 2.
					if `obj2' < `obj3' {
						
						* Quantile regression with "set2"
						qreg lnw `var_qreg' if d_miss == 0 & set2 == 1 [pw = wgt], q(`q') wlsiter(2)
						
					}
					drop set2 set3 yhat2 yhat3
					
				}
				else {
					*-----------------------------------------------------------
					* Case 4: Double censoring
					*-----------------------------------------------------------
					
					*=====================================
					* Step 2: Censored Quantile Regression
					*=====================================
					* Probability of not being censored
					gen no_pr_c2 = 1 - pr_c2  if d_miss == 0
					* Subsample selection 1:
					* The quantile of interest should fall inside the
					*  range of probabilities of not being censored.
					gen set2     = (pr_c1 < `q' & `q' < no_pr_c2)  if d_miss == 0
					
					* Subsample selection 2 (refinement):
					* Reduce the probability of not being right-censored by
					*  the 0.1th quantiles of empirical probability in set2
					*  to accommodate possible specification errors.
					* Note that if the probability of not being censored is
					*  almost one, keep all observations.
					centile pr_c1         if d_miss == 0 & set2 == 1, c(10)
					local   pc2 = r(c_1)  
					centile no_pr_c2      if d_miss == 0 & set2 == 1, c(10)
					local   no_pc2 = r(c_1)
					
					* Stop dropping observations if the probability of not being
					*  censored is very large above "0.999".
					if `no_pc2' < 0.999 {
						replace set2 = 0 if d_miss == 0 & set2 == 1 & no_pr_c2 < `no_pc2'
					}
					* Stop dropping observations if the probability of being
					*  censored is very small above "0.001".
					if `pc2' > 0.001 {
						replace set2 = 0 if d_miss == 0 & set2 == 1 & pr_c1 < `pc2'
					}
					
					* Quantile regression with selected sample 
					qreg lnw `var_qreg' if d_miss == 0 & set2 == 1 [pw = wgt], q(`q') wlsiter(2)
					
					* Fitted values 
					predict double yhat2 if d_miss == 0, xb
					
					* Value of the objective function in step 2
					gen      resid2 = lnw - yhat2 if d_miss == 0
					replace  resid2 = lnw - lnmw  if d_miss == 0 & yhat2 < lnmw 
					replace  resid2 = lnw - lntop if d_miss == 0 & lntop < yhat2 
					gen      vecR2  = ( `q' - 1 * (resid2 <= 0) ) * resid2  if d_miss == 0
					sum      vecR2 [aw = wgt]  if d_miss == 0   /* <== weighting is needed */
					local    obj2   = r(sum)
					drop     resid2 vecR2
					
					*======================================
					* Step 3: Censored quantile regression
					*======================================
					* Subsample selection 3:
					* Keep observations for which fitted values (i.e., predicted 
					*  probability of being censored) is above 0.03th quantile
					*  and below 0.97th quantile in selected sample.					
					centile yhat2 if d_miss == 0 & lnmw < yhat2,  c(3) 
					local   y_cut3A = r(c_1)
					centile yhat2 if d_miss == 0 & yhat2 < lntop, c(97)
					local   y_cut3B = r(c_1)
					gen     set3    = (`y_cut3A' < yhat2 & yhat2 < `y_cut3B')  if d_miss == 0
					
					* Quantile regression with selected sample
					qreg lnw `var_qreg' if d_miss == 0 & set3 == 1 [pw = wgt], q(`q') wlsiter(1)
					
					* Fitted values 
					predict double yhat3 if d_miss == 0, xb
					
					* Value of the objective function in step 3
					gen      resid3 = lnw - yhat3 if d_miss == 0
					replace  resid3 = lnw - lnmw  if d_miss == 0 & yhat3 < lnmw 
					replace  resid3 = lnw - lntop if d_miss == 0 & lntop < yhat3 
					gen      vecR3   = ( `q' - 1 * (resid3 <= 0) ) * resid3  if d_miss == 0
					sum      vecR3 [aw = wgt]  if d_miss == 0 /* weighting is needed */
					local    obj3   = r(sum)
					drop     resid3 vecR3
					
					* Check if the model fit improves in step 3. If it does not,
					* redo step 2.
					if `obj2' < `obj3' {
						
						* Quantile regression with "set2"
						qreg lnw `var_qreg' if d_miss == 0 & set2 == 1 [pw = wgt], q(`q') wlsiter(2)
						
					}
					drop no_pr_c2 set2 yhat2 set3 yhat3
	
				}
			}
			
			*******************
			* Result
			*******************
			* Store fitted values for individuals with missing wages
			predict double lnw_q`Q' if d_miss == 1, xb
						
		} /* EoL: quantile */
		
		
		*=======================
		* Imputation of wages 
		*=======================
		* To impute missing observations, we create 10 copies of missing observations 
		expand 10 if d_miss == 1
		sort id
		by id: gen obs = _n
		
		* Replace missing wages with predicted wages
		local thre1 = `thre' - 1
		gen pct = round( 4 + (`thre1' - 4) * uniform() )
		forvalues Q = 4/`thre1' {
			
			replace lnw = lnw_q`Q' if d_miss == 1 & pct == `Q'
			
		}

		*================================
		* Imputation of hours and weight
		*================================
		gen lnw2 = lnw^2
		gen lnw3 = lnw^3
		gen lnw4 = lnw^4
		gen lnw5 = lnw^5
		* Regression without weight. 
		reg h lnw lnw2-lnw5 `var_qreg' if d_miss == 0
		predict h_pred
		* Calculate weight = sampling weight * hours (or imputed hours) / 10,
		*  where 10 is coming from the number of times imputation. 
		replace wgt = wgt0 * h_pred / 10 if d_miss == 1
		drop h_pred
		replace wgt = 0 if d_miss == 1 & wgt<0
		
		*=======================================================================
		* Imputed observations
		*=======================================================================
		* We set d=1 for imputed observations.
		* Note that hourly paid workers are not top-coded.
		replace d   = 1 if d_miss == 1                /* hourly-wage earner */
		replace dc2 = 0 if d_miss == 1                /* no top coding      */
		replace dc1 = 1 if d_miss == 1 & lnw <= lnmw
		replace lnw = lnmw if d_miss == 1 & lnw <= lnmw
		
		**********************
		* Save 
		**********************
		keep id obs lnw wgt case_step1 dc1 dc2
		if `thre'!=0 {
			save `dir'\data\est\step0_`t'_`j'.dta, replace
		}
		if `thre'==0 {
			save `dir'\data\est\est0\step0_`t'_`j'.dta, replace
		}
		
	} /* EoL: state */
	
} /* EoL: time */	
